import re
import os
import papis.downloaders.base
import arxiv2bib


class Downloader(papis.downloaders.base.Downloader):

    def __init__(self, url):
        papis.downloaders.base.Downloader.__init__(self, url)

    @classmethod
    def match(cls, url):
        """
        >>> Downloader.match(\
                'blah://pubs.arxiv.org/doi/abs/10.1021/acs.jchemed.6b00559'\
            ) is False
        False
        >>> Downloader.match(\
                'blah://pubs.arxiv.org/!@#!@$!%!@%!$che.6b00559'\
            ) is False
        False
        >>> Downloader.match(\
                'arxiv.com/!@#!@$!%!@%!$chemed.6b00559'\
            ) is False
        True
        >>> Downloader.match('arXiv:1701.08223v2') is False
        False
        >>> import re; re.match(\
                r".*\.pdf\.pdf$",\
                Downloader.match('https://arxiv.org/pdf/1707.09820.pdf').getDocumentUrl()\
            ) is None
        True
        """
        m = re.match(r"^arxiv:(.*)", url, re.IGNORECASE)
        if m:
            url = "https://arxiv.org/abs/{m}".format(m=m.group(1))
        if re.match(r".*arxiv.org.*", url):
            # https://arxiv.org/pdf/1707.09820|.pdf?blahslas=sdfad|
            url = re.sub(r"\.pdf.*$", "", url)
            return Downloader(url)
        else:
            return False

    def get_identifier(self):
        """Get arxiv identifier
        :returns: Identifier
        """
        url = self.getUrl()
        return re.sub(r'^.*arxiv.org/(abs|pdf)/(.*)\/?$', r'\2', url)

    def getBibtexUrl(self):
        identifier = self.get_identifier()
        return identifier

    def downloadBibtex(self):
        bibtexCli = arxiv2bib.Cli([self.getBibtexUrl()])
        bibtexCli.run()
        data = os.linesep.join(bibtexCli.output)
        self.bibtex_data = data

    def getDocumentUrl(self):
        # https://arxiv.org/pdf/1702.01590
        url = self.getUrl()
        identifier = self.get_identifier()
        self.logger.debug("[paper id] = %s" % identifier)
        pdf_url = "https://arxiv.org/pdf/"+identifier+".pdf"
        self.logger.debug("[pdf url] = %s" % pdf_url)
        return pdf_url

# vim-run: python3 -m doctest %



import re
import papis.downloaders.base


class Downloader(papis.downloaders.base.Downloader):

    def __init__(self, url):
        papis.downloaders.base.Downloader.__init__(self, url)

    @classmethod
    def match(cls, url):
        # http://aip.scitation.org/doi/10.1063/1.4873138
        if re.match(r".*(aip|aapt).scitation.org.*", url):
            return Downloader(url)
        else:
            return False

    def getDoi(self):
        mdoi = re.match(r'.*scitation.org/doi/(.*)', self.getUrl())
        if mdoi:
            doi = mdoi.group(1).replace("abs/", "").replace("full/", "")
            self.logger.debug("[doi] = %s" % doi)
            return doi
        else:
            self.logger.error("Doi not found!!")

    def getDocumentUrl(self):
        # http://aip.scitation.org/doi/pdf/10.1063/1.4873138
        durl = "http://aip.scitation.org/doi/pdf/%s" % self.getDoi()
        self.logger.debug("[doc url] = %s" % durl)
        return durl

    def getBibtexUrl(self):
        url = "http://aip.scitation.org/action/downloadCitation"\
              "?format=bibtex&cookieSet=1&doi=%s" % self.getDoi()
        self.logger.debug("[bibtex url] = %s" % url)
        return url



import re
import bs4
import papis.downloaders.base
import urllib.request


class Downloader(papis.downloaders.base.Downloader):

    def __init__(self, url):
        papis.downloaders.base.Downloader.__init__(self, url)

    @classmethod
    def match(cls, url):
        # http://libgen.io/ads.php?md5=CBA569C45B32CA3DF52E736CD8EF6340
        if re.match(r".*libgen.*", url):
            return Downloader(url)
        else:
            return False

    def downloadBibtex(self):
        raw_data = urllib.request.urlopen(self.getUrl())\
            .read()\
            .decode('utf-8')
        soup = bs4.BeautifulSoup(raw_data, "html.parser")
        textareas = soup.find_all("textarea")
        if not textareas:
            return False
        self.bibtex_data = textareas[0].text



import os
import logging
import urllib.request
import papis.config


class Downloader(object):

    """This is the base class for every downloader.
    """

    def __init__(self, url=""):
        self.url = url
        self.src = os.path.basename(__file__)
        self.logger = logging.getLogger("downloaders:"+self.src)
        self.bibtex_data = None
        self.document_data = None
        self.logger.debug("[url] = %s" % url)

    @classmethod
    def match(url):
        """This method should be called to know if a given url matches
        the downloader or not.

        For example, a valid match for archive would be:
        .. code:: python

            return re.match(r".*arxiv.org.*", url)

        it will return something that is true if it matches and something
        falsely otherwise.

        :param url: Url where the document should be retrieved from.
        :type  url: str
        """
        raise NotImplementedError(
            "Matching url not implemented for this downloader"
        )

    def getBibtexUrl(self):
        """It returns the urls that is to be access to download
        the bibtex information. It has to be implemented for every
        downloader, or otherwise it will raise an exception.

        :returns: Bibtex url
        :rtype:  str
        """
        raise NotImplementedError(
            "Getting bibtex url not implemented for this downloader"
        )

    def getBibtexData(self):
        """Get the bibtex_data data if it has been downloaded already
        and if not download it and return the data in utf-8 format.

        :returns: Bibtex data in utf-8 format
        :rtype:  str
        """
        if not self.bibtex_data:
            self.downloadBibtex()
        return self.bibtex_data

    def downloadBibtex(self):
        """Bibtex downloader, it should try to download bibtex information from
        the url provided by ``getBibtexUrl``.

        It sets the ``bibtex_data`` attribute if it succeeds.

        :returns: Nothing
        :rtype:  None
        """
        url = self.getBibtexUrl()
        if not url:
            return False
        data = urllib.request.urlopen(url)\
            .read()\
            .decode('utf-8')
        self.bibtex_data = data

    def getDocumentUrl(self):
        """It returns the urls that is to be access to download
        the document information. It has to be implemented for every
        downloader, or otherwise it will raise an exception.

        :returns: Document url
        :rtype:  str
        """
        raise NotImplementedError(
            "Getting bibtex url not implemented for this downloader"
        )

    def getDoi(self):
        """It returns the doi of the document, if it is retrievable.
        It has to be implemented for every downloader, or otherwise it will
        raise an exception.

        :returns: Document doi
        :rtype:  str
        """
        raise NotImplementedError(
            "Getting document url not implemented for this downloader"
        )

    def getDocumentData(self):
        """Get the document_data data if it has been downloaded already
        and if not download it and return the data in binary format.

        :returns: Document data in binary format
        :rtype:  str
        """
        if not self.document_data:
            self.downloadDocument()
        return self.document_data

    def downloadDocument(self):
        """Document downloader, it should try to download document information from
        the url provided by ``getDocumentUrl``.

        It sets the ``document_data`` attribute if it succeeds.

        :returns: Nothing
        :rtype:  None
        """
        self.logger.debug("Downloading document")
        url = self.getDocumentUrl()
        if not url:
            return False
        request = urllib.request.Request(
            url,
            headers={
                'User-Agent': papis.config.get('user-agent')
            }
        )
        data = urllib.request.urlopen(request).read()
        self.document_data = data

    def setUrl(self, url):
        """Url setter for Downloader

        :param url: String containing a valid url
        :type  url: str
        :returns: Downloader object
        """
        self.url = url
        return self

    def getUrl(self):
        """Url getter for Downloader
        :returns: Main url of the Downloader
        :rtype:  str
        """
        return self.url



import papis
import os
import sys
import re
import tempfile
import hashlib
import shutil
import string
import subprocess
import papis.api
import papis.utils
import papis.config
import papis.bibtex
import papis.document
import papis.downloaders.utils

class Command(papis.commands.Command):

    def init(self):

        self.parser = self.get_subparsers().add_parser(
            "add",
            help="Add a document into a given library"
        )

        self.parser.add_argument(
            "document",
            help="Document file name",
            default=[],
            nargs="*",
            action="store"
        )

        self.parser.add_argument(
            "-d", "--dir",
            help="Subfolder in the library",
            default="",
            action="store"
        )

        self.parser.add_argument(
            "-i", "--interactive",
            help="Do some of the actions interactively",
            action='store_false' if papis.config.get('add-interactive') \
                else 'store_true'
        )

        self.parser.add_argument(
            "--name",
            help="Name for the main folder",
            default="",
            action="store"
        )

        self.parser.add_argument(
            "--title",
            help="Title for document",
            default="",
            action="store"
        )

        self.parser.add_argument(
            "--author",
            help="Author(s) for document",
            default="",
            action="store"
        )

        self.parser.add_argument(
            "--from-bibtex",
            help="Parse information from a bibtex file",
            default="",
            action="store"
        )

        self.parser.add_argument(
            "--from-yaml",
            help="Parse information from a yaml file",
            default="",
            action="store"
        )

        self.parser.add_argument(
            "--from-url",
            help="Get document and information from a"
                "given url, a parser must be implemented",
            default="",
            action="store"
        )

        self.parser.add_argument(
            "--from-doi",
            help="Doi to try to get information from",
            default=None,
            action="store"
        )

        self.parser.add_argument(
            "--from-vcf",
            help="Get contact information from a vcard (.vcf) file",
            default=None,
            action="store"
        )

        self.parser.add_argument(
            "--to",
            help="When --to is specified, the document will be added to the"
                "selected already existing document entry.",
            nargs="?",
            action="store"
        )

        self.parser.add_argument(
            "--confirm",
            help="Ask to confirm before adding to the collection",
            action='store_false' if papis.config.get('add-confirm') \
                else 'store_true'
        )

        self.parser.add_argument(
            "--open",
            help="Open file before adding document",
            action='store_false' if papis.config.get('add-open') \
                else 'store_true'
        )

        self.parser.add_argument(
            "--edit",
            help="Edit info file before adding document",
            action='store_false' if papis.config.get('add-edit') \
                else 'store_true'
        )

        self.parser.add_argument(
            "--commit",
            help="Commit document if library is a git repository",
            action="store_true"
        )

        self.parser.add_argument(
            "--no-document",
            help="Add entry without a document related to it",
            action="store_true"
        )

    def get_hash_folder(self, data, document_path):
        """Folder name where the document will be stored.

        :data: Data parsed for the actual document
        :document_path: Path of the document

        """
        author = "-{:.20}".format(data["author"])\
                 if "author" in data.keys() else ""
        fd = open(document_path, "rb")
        md5 = hashlib.md5(fd.read(4096)).hexdigest()
        fd.close()
        result = re.sub(r"[\\'\",.(){}]", "", md5 + author)\
                   .replace(" ", "-")
        return result

    def get_document_extension(self, documentPath):
        """Get document extension

        :document_path: Path of the document
        :returns: Extension (string)

        """
        # TODO: mimetype based (mimetype, rifle, ranger-fm ...?)
        m = re.match(r"^(.*)\.([a-zA-Z0-9]*)$", os.path.basename(documentPath))
        extension = m.group(2) if m else "txt"
        self.logger.debug("[ext] = %s" % extension)
        return extension

    def get_meta_data(self, key, document_path):
        # TODO: Consistent a general way to get metadata from documents.
        # TODO: pdfminer does not work very well
        self.logger.debug("Retrieving %s meta data" % key)
        extension = self.get_document_extension(document_path)
        return None

    def get_default_title(self, data, document_path):
        if "title" in data.keys(): return data["title"]
        extension = self.get_document_extension(document_path)
        title = self.get_meta_data("title", document_path)
        if not title:
            title = os.path.basename(document_path)\
                            .replace("."+extension, "")\
                            .replace("_", " ")\
                            .replace("-", " ")
            if self.get_args().interactive:
                title = papis.utils.input(
                    'Title?', title
                )
        return title

    def get_default_author(self, data, document_path):
        if "author" in data.keys(): return data["author"]
        author = self.get_meta_data("author", document_path)
        if not author:
            author = "Unknown"
            if self.get_args().interactive:
                author = papis.utils.input(
                    'Author?', author
                )
        return author

    def init_contact_mode(self):
        """Initialize the contact mode
        """
        self.logger.debug("Initialising contact mode")
        self.args.document = [papis.utils.get_info_file_name()]
        self.args.from_yaml = papis.utils.get_info_file_name()
        if os.path.exists(self.args.document[0]):
            return True
        self.args.edit = True
        self.args.confirm = True
        template = papis.document.Document.get_vcf_template()
        with open(self.args.document[0], "w+") as fd:
            fd.write(template)

    def main(self):
        if papis.config.in_mode("contact"):
            self.init_contact_mode()
        lib_dir = os.path.expanduser(papis.config.get('dir'))
        data = dict()
        out_folder_name = None
        in_documents_paths = self.args.document
        in_documents_names = []
        temp_dir = tempfile.mkdtemp("-"+self.args.lib)
        if self.args.from_url:
            url_data = papis.downloaders.utils.get(self.args.from_url)
            data.update(url_data["data"])
            in_documents_paths.extend(url_data["documents_paths"])
            # If no data was retrieved and doi was found, try to get
            # information with the document's doi
            self.logger.warning(
                "I could not get any data from %s" % self.args.from_url
            )
            if not data and url_data["doi"] is not None and\
                not self.args.from_doi:
                self.args.from_doi = url_data["doi"]
        if self.args.from_bibtex:
            bib_data = papis.bibtex.bibtex_to_dict(self.args.from_bibtex)
            if len(bib_data) > 1:
                self.logger.warning(
                    'Your bibtex file contains more than one entry,'
                    ' I will be taking the first entry'
                )
            data.update(bib_data[0])
        if self.args.from_doi:
            self.logger.debug("I'll try using doi %s" % self.args.from_doi)
            data.update(papis.utils.doi_to_data(self.args.from_doi))
            if len(self.get_args().document) == 0 and \
                    papis.config.get('doc-url-key-name') in data.keys():
                doc_url = data[papis.config.get('doc-url-key-name')]
                self.logger.info(
                    'I am trying to download the document from %s' % doc_url
                )
                down = papis.downloaders.utils.get_downloader(
                    doc_url,
                    'get'
                )
                file_name = tempfile.mktemp()
                with open(file_name, 'wb+') as fd:
                    fd.write(down.getDocumentData())
                self.logger.info('Opening the file')
                papis.api.open_file(file_name)
                if papis.utils.confirm('Do you want to use this file?'):
                    self.args.document.append(file_name)

        if self.args.from_yaml:
            self.logger.debug("Yaml input file = %s" % self.args.from_yaml)
            data.update(papis.utils.yaml_to_data(self.args.from_yaml))
        if self.args.from_vcf:
            data.update(papis.utils.vcf_to_data(self.args.from_vcf))
        in_documents_names = [
            papis.utils.clean_document_name(doc_path)
            for doc_path in in_documents_paths
        ]

        # Decide if we are adding the documents to an already existing document
        # or it is a new document
        if self.args.to:
            self.logger.debug(
                "Searching for the document where to add the files"
            )
            documents = papis.api.get_documents_in_dir(
                lib_dir,
                self.args.to
            )
            document = self.pick(documents) or sys.exit(0)
            document.update(
                data,
                interactive=self.args.interactive
            )
            document.save()
            data = document.to_dict()
            in_documents_paths = [
                os.path.join(
                    document.get_main_folder(),
                    d
                ) for d in document.get_files()
            ] + in_documents_paths
            data["files"] = document.get_files() + in_documents_names
            # set out folder name the folder of the found document
            out_folder_name = document.get_main_folder_name()
            out_folder_path = document.get_main_folder()
        else:
            document = papis.document.Document(temp_dir)
            if not papis.config.in_mode("contact"):
                if len(in_documents_paths) == 0:
                    if not self.get_args().no_document:
                        self.logger.error("No documents to be added")
                        sys.exit(1)
                    else:
                        in_documents_paths = [document.get_info_file()]
                        # We need the names to add them in the file field
                        # in the info file
                        in_documents_names = [papis.utils.get_info_file_name()]
                        # Save document to create the info file
                        document.update(
                            data, force=True, interactive=self.args.interactive
                        )
                        document.save()
                data["title"] = self.args.title or self.get_default_title(
                    data,
                    in_documents_paths[0]
                )
                data["author"] = self.args.author or self.get_default_author(
                    data,
                    in_documents_paths[0]
                )
                self.logger.debug("Author = % s" % data["author"])
                self.logger.debug("Title = % s" % data["title"])
            if not self.args.name:
                self.logger.debug("Getting an automatic name")
                out_folder_name = self.get_hash_folder(
                    data,
                    in_documents_paths[0]
                )
            else:
                out_folder_name = string\
                            .Template(self.args.name)\
                            .safe_substitute(data)\
                            .replace(" ", "-")
            data["files"] = in_documents_names
            out_folder_path = os.path.join(
                lib_dir, self.args.dir,  out_folder_name
            )

        self.logger.debug("Folder name = % s" % out_folder_name)
        self.logger.debug("Folder path = % s" % out_folder_path)
        self.logger.debug("File(s)     = % s" % in_documents_paths)

        # Create folders if they do not exists.
        if not os.path.isdir(temp_dir):
            self.logger.debug("Creating directory '%s'" % temp_dir)
            os.makedirs(temp_dir)

        # Check if the user wants to edit before submitting the doc
        # to the library
        if self.args.edit:
            document.update(
                data, force=True, interactive=self.args.interactive
            )
            document.save()
            self.logger.debug("Editing file before adding it")
            papis.api.edit_file(document.get_info_file())
            self.logger.debug("Loading the changes made by editing")
            document.load()
            data = document.to_dict()

        # First prepare everything in the temporary directory
        for i in range(min(len(in_documents_paths), len(data["files"]))):
            in_doc_name = data["files"][i]
            in_file_path = in_documents_paths[i]
            assert(os.path.exists(in_file_path))
            endDocumentPath = os.path.join(
                document.get_main_folder(),
                in_doc_name
            )
            if os.path.exists(endDocumentPath):
                self.logger.debug(
                    "%s already exists, ignoring..." % endDocumentPath
                )
                continue
            if not self.args.no_document:
                self.logger.debug(
                    "[CP] '%s' to '%s'" %
                    (in_file_path, endDocumentPath)
                )
                shutil.copy(in_file_path, endDocumentPath)

        document.update(data, force=True)
        if self.get_args().open:
            for d_path in in_documents_paths:
                papis.api.open_file(d_path)
        if self.args.confirm:
            if not papis.utils.confirm('Really add?'):
                return 0
        document.save()
        if self.args.to:
            sys.exit(0)
        self.logger.debug(
            "[MV] '%s' to '%s'" %
            (document.get_main_folder(), out_folder_path)
        )
        shutil.move(document.get_main_folder(), out_folder_path)
        papis.api.clear_lib_cache()
        if self.args.commit and papis.utils.lib_is_git_repo(self.args.lib):
            subprocess.call(["git", "-C", out_folder_path, "add", "."])
            subprocess.call(
                ["git", "-C", out_folder_path, "commit", "-m", "Add document"]
            )



from papis.gui.tk import PapisWidget
from papis.gui.tk import tk
import re


class Command(object):

    def __init__(self, name, function, prompt, nargs=0):
        self.name = name
        self.function = function
        self.nargs = nargs
        self.args = []
        self.prompt = prompt

    def set_args(self, args):
        self.args = args

    def run(self):
        return self.function(*self.args)


class Prompt(tk.Text, PapisWidget):

    def __init__(self, *args, **kwargs):
        tk.Text.__init__(self, *args, **kwargs)
        PapisWidget.__init__(self)
        self.bind("<Control-u>", self.clear)
        self.command = ""
        self.last_command = ""
        self.registered_commands = []
        self.reset_style()

    def reset_style(self):
        self["font"] = self.get_config("prompt-font-size")
        self["fg"] = self.get_config("prompt-fg")
        self["insertbackground"] = self.get_config("insertbackground")
        self["bg"] = self.get_config("prompt-bg")
        self["borderwidth"] = -1
        self["cursor"] = self.get_config("cursor")
        self["height"] = 1

    def changed(self):
        self.get_command()
        if self.last_command == self.command:
            return False
        else:
            return True

    def register_command(self, cmd):
        self.registered_commands.append(cmd)

    def get_registered_commands(self):
        return self.registered_commands

    def parse_command(self, command_string):
        for cmd in self.get_registered_commands():
            m = re.match(r"^\s*%s\s*" % cmd.name, command_string)
            if m:
                return cmd
        return None

    def run(self, event=None):
        command_string = self.get_command()
        command = self.parse_command(command_string)
        if command is None:
            self.logger.error("Command %s not recognised" % command_string)
            return False
        self.logger.debug("Running %s" % command_string)
        return command.run()

    def add_new_command(self, name, function, nargs=0):
        self.logger.debug("Adding command %s" % name)
        cmd = Command(name, function, self, nargs)
        self.register_command(cmd)

    def get_command(self):
        self.last_command = self.command
        if self.get_mode() == self.command_mode:
            self.command = self.get(1.0, tk.END)
        return self.command

    def echomsg(self, text):
        self.clear()
        self["height"] = len(text.split("\n"))-1
        self.insert(1.0, text)

    def echoerr(self, text):
        self.clear()
        self["height"] = len(text.split("\n"))-1
        self.insert(1.0, text)

    def clear(self, event=None):
        self["height"] = 1
        self.delete(1.0, tk.END)

    def focus(self, event=None):
        self.set_mode(self.command_mode)
        self.focus_set()

    def autocomplete(self, event=None):
        pass



"""This command will try its best to find a source in the internet for the
document at hand.

Of course if the document has an url key in its info file, it will use this url
to open it in a browser.  Also if it has a doc_url key, or a doi, it will try
to compose urls out of these to open it.

If none of the above work, then it will try to use a search engine with the
document's information (using the ``browse-query-format``).  You can select
wich search engine you want to use using the ``search-engine`` setting.

"""
import papis
import os
import sys
import papis.utils
import papis.config


class Command(papis.commands.Command):
    def init(self):

        self.parser = self.get_subparsers().add_parser(
            "browse",
            help="Open document's url in a browser"
        )

        self.add_search_argument()

    def main(self):
        documents = papis.api.get_documents_in_lib(
            self.get_args().lib,
            self.get_args().search
        )
        document = self.pick(documents) or sys.exit(0)

        url = None
        if "url" in document.keys():
            url = document["url"]
        elif 'doi' in document.keys():
            url = 'https://doi.org/' + document['doi']
        elif papis.config.get('doc-url-key-name') in document.keys():
            url = document[papis.config.get('doc-url-key-name')]
        else:
            from urllib.parse import urlencode
            params = {
                'q': papis.utils.format_doc(
                    papis.config.get('browse-query-format'),
                    document
                )
            }
            url = papis.config.get('search-engine') + '/?' + urlencode(params)


        if url is None:
            self.logger.warning(
                "No url for %s possible" % (document.get_main_folder_name())
            )
        else:
            self.logger.debug("Opening url %s:" % url)
            papis.utils.general_open(
                url, "browser"
            )



import os
import sys
import papis
import papis.api
import papis.config
import papis.commands
import logging


class Command(papis.commands.Command):

    def init(self):

        self.default_parser.add_argument(
            "-v",
            "--verbose",
            help="Make the output verbose (equivalent to --log DEBUG)",
            default=False,
            action="store_true"
        )

        self.default_parser.add_argument(
            "-V",
            "--version",
            help="Show version number",
            default=False,
            action="store_true"
        )

        self.default_parser.add_argument(
            "-l",
            "--lib",
            help="Choose a library name or library path (unamed library)",
            default=papis.config.get("default-library"),
            action="store"
        )

        self.default_parser.add_argument(
            "-c",
            "--config",
            help="Configuration file to use",
            default=None,
            action="store"
        )

        self.default_parser.add_argument(
            "--log",
            help="Logging level",
            choices=[
                "INFO",
                "DEBUG",
                "WARNING",
                "ERROR",
                "CRITICAL"
                ],
            action="store",
            default="INFO"
        )

        self.default_parser.add_argument(
            "--picktool",
            help="Override picktool",
            action="store",
            default=""
        )

        self.default_parser.add_argument(
            "--pick-lib",
            help="Pick library to use",
            action="store_true"
        )

        self.default_parser.add_argument(
            "--clear-cache", "--cc",
            help="Clear cache of the library used",
            action="store_true"
        )

        self.default_parser.add_argument(
            "-j", "--cores",
            help="Number of cores to run some multicore functionality",
            type=int,
            default=__import__("multiprocessing").cpu_count(),
            action="store"
        )

        self.default_parser.add_argument(
            "--set",
            help="Set key value, e.g., "
                 "--set info-name=information.yaml,opentool=evince",
            action="store"
        )

    def main(self):
        self.set_args(papis.commands.get_args())
        log_format = '%(levelname)s:%(name)s:%(message)s'
        if self.args.verbose:
            self.args.log = "DEBUG"
            log_format = '%(relativeCreated)d-'+log_format
        logging.basicConfig(
            level=getattr(logging, self.args.log),
            format=log_format
        )

        if self.args.version:
            print('Papis - %s' % papis.__version__)
            sys.exit(0)

        if self.args.set:
            key_vals = [d.split("=") for d in self.args.set.split(",")]
            for pair in key_vals:
                key = pair[0]
                val = pair[1]
                papis.config.set(key, val)

        if self.args.config:
            papis.config.set_config_file(self.args.config)
            papis.config.reset_configuration()
            papis.commands.Command.config = papis.config.get_configuration()

        if self.args.picktool:
            papis.config.set("picktool", self.args.picktool)

        if self.args.pick_lib:
            self.args.lib = papis.api.pick(
                papis.api.get_libraries(),
                pick_config=dict(header_filter=lambda x: x)
            )

        if self.args.lib not in self.get_config().keys():
            if os.path.exists(self.args.lib):
                # Check if the path exists, then use this path as a new library
                self.logger.debug("Using library %s" % self.args.lib)
                self.get_config()[self.args.lib] = dict()
                self.get_config()[self.args.lib]["dir"] = self.args.lib
            else:
                self.logger.error(
                    "Library '%s' does not seem to exist" % self.args.lib
                )
                sys.exit(1)

        if self.args.clear_cache:
            papis.api.clear_lib_cache(self.args.lib)

        commands = papis.commands.get_commands()

        if self.args.command:
            if self.args.command in commands.keys():
                commands[self.args.command].set_args(self.args)
                commands[self.args.command].main()



import papis
import os
import sys
import shutil
import papis.utils


class Command(papis.commands.Command):
    def init(self):

        self.parser = self.get_subparsers().add_parser(
            "export",
            help="""Export a document from a given library"""
        )

        self.add_search_argument()

        self.parser.add_argument(
            "--yaml",
            help="Export into bibtex",
            default=False,
            action="store_true"
        )

        self.parser.add_argument(
            "--bibtex",
            help="Export into bibtex",
            default=False,
            action="store_true"
        )

        self.parser.add_argument(
            "--folder",
            help="Export document folder to share",
            default=False,
            action="store_true"
        )

        self.parser.add_argument(
            "--no-bibtex",
            help="When exporting to a folder, do not include the bibtex",
            default=False,
            action="store_true"
        )

        self.parser.add_argument(
            "-o",
            "--out",
            help="Outfile or outdir",
            default="",
            action="store"
        )

        self.parser.add_argument(
            "-t",
            "--text",
            help="Text formated reference",
            action="store_true"
        )

        self.parser.add_argument(
            "-a", "--all",
            help="Export all without picking",
            action="store_true"
        )

        self.parser.add_argument(
            "--vcf",
            help="Export contact to vcf format",
            action="store_true"
        )

    def export(self, document):
        """Main action in export command
        """
        folder = document.get_main_folder()
        if not self.args.folder and not self.args.out:
            self.args.out = "/dev/stdout"
        if self.args.bibtex:
            print(document.to_bibtex())
        if self.args.text:
            text_format = papis.config.get('export-text-format')
            text = papis.utils.format_doc(text_format, document)
            open(self.args.out, "w").write(text)
        elif self.args.folder:
            outdir = self.args.out or document.get_main_folder_name()
            shutil.copytree(folder, outdir)
            if not self.args.no_bibtex:
                open(
                    os.path.join(outdir, "info.bib"),
                    "w+"
                ).write(document.to_bibtex())
        elif self.args.yaml:
            open(self.args.out, "w").write(document.dump())
        elif self.args.vcf:
            open(self.args.out, "w").write(document.to_vcf())
        else:
            pass

    def main(self):

        documents = papis.api.get_documents_in_lib(
            self.get_args().lib,
            self.get_args().search
        )

        if not self.args.all:
            document = self.pick(documents) or sys.exit(0)
            documents = [document]

        for document in documents:
            self.export(document)



import sys
import os
import re
import configparser
import papis.commands


class Command(papis.commands.Command):
    def init(self):

        self.parser = self.get_subparsers().add_parser(
            "config",
            help="Print configuration values"
        )

        self.parser.add_argument(
            "option",
            help="Variable to print",
            default="",
            action="store"
        )

    def main(self):
        option = self.args.option.split(".")
        self.logger.debug(option)
        if len(option) == 1:
            key = option[0]
            section = None
        elif len(option) == 2:
            section = option[0]
            key = option[1]
        self.logger.debug("key = %s, sec = %s" % (key, section))
        val = papis.config.get(key, section=section)
        print(val)



import papis.utils
import papis.document
import papis.config
import papis.bibtex
import urllib.request


class Command(papis.commands.Command):

    def init(self):

        self.parser = self.get_subparsers().add_parser(
            "explore",
            help="Explore on the internet"
        )

        self.parser.add_argument(
            "search",
            help="Search string",
            default=[],
            nargs="*",
            action="store"
        )

        self.parser.add_argument(
            "--arxiv",
            help="Search on the arxiv",
            action="store_true"
        )

        self.parser.add_argument(
            "--max",
            help="Maximum number of items",
            default=30,
            action="store"
        )

    def arxiv(self, search):
        # FIXME: use a more lightweight library than bs4, it needs some time
        # to import the module
        import bs4
        main_url = "http://export.arxiv.org/api/query?search_query="
        url = main_url+"all:{}&max_results={}".format(
            "%20".join(search),
            self.args.max
        )
        self.logger.debug("Url = %s" % url)
        raw_data = urllib.request.urlopen(url).read().decode('utf-8')
        soup = bs4.BeautifulSoup(raw_data, "html.parser")
        entries = soup.find_all("entry")
        self.logger.debug("%s matches" % len(entries))
        documents = []
        for entry in entries:
            data = dict()
            data["abstract"] = entry.find("summary").get_text().replace(
                "\n", " "
            )
            data["url"] = entry.find("id").get_text()
            data["year"] = entry.find("published").get_text()[0:4]
            data["title"] = entry.find("title").get_text().replace("\n", " ")
            data["author"] = ", ".join(
                [
                    author.get_text().replace("\n", "")
                    for author in entry.find_all("author")
                ]
            )
            document = papis.document.Document(data=data)
            documents.append(document)
        self.pick(documents)

    def main(self):
        if self.arxiv:
            self.arxiv(self.args.search)
        else:
            self.arxiv(self.args.search)



import os
import re
import argparse
import subprocess
import papis.config
import papis.commands


class Command(papis.commands.Command):

    def init(self, path):
        self.script_path = path

        self.parser = self.get_subparsers().add_parser(
            self.get_command_name(),
            help=self.get_command_help()
        )

        self.parser.add_argument(
            "args",
            help="Arguments",
            nargs=argparse.REMAINDER,
            action="store"
        )

    def get_command_name(self):
        m = re.match(r"^.*papis-(.*)$", self.script_path)
        return m.group(1) if m else None

    def get_command_help(self):
        magic_word = papis.config.get("scripts-short-help-regex")
        with open(self.script_path) as fd:
            for line in fd:
                m = re.match(magic_word, line)
                if m:
                    return m.group(1)
        return "No help message available"

    def export_variables(self):
        """Export environment variables so that external script can access to
        the information
        """
        os.environ["PAPIS_LIB"] = self.args.lib
        os.environ["PAPIS_LIB_PATH"] = papis.config.get('dir')
        os.environ["PAPIS_CONFIG_PATH"] = papis.config.get_config_folder()
        os.environ["PAPIS_CONFIG_FILE"] = papis.config.get_config_file()
        os.environ["PAPIS_SCRIPTS_PATH"] = papis.config.get_scripts_folder()
        os.environ["PAPIS_VERBOSE"] = "-v" if self.args.verbose else ""

    def main(self):
        self.export_variables()
        subprocess.call([self.script_path] + self.args.args)



import re
import logging
import os

logger = logging.getLogger("bibtex")

bibtex_types = [
  "article",
  "book",
  "booklet",
  "conference",
  "inbook",
  "incollection",
  "inproceedings",
  "manual",
  "mastersthesis",
  "misc",
  "phdthesis",
  "proceedings",
  "techreport",
  "unpublished"
]

bibtex_keys = [
  "address",
  "annote",
  "author",
  "booktitle",
  "doi",
  "chapter",
  "crossref",
  "edition",
  "editor",
  "howpublished",
  "institution",
  "journal",
  "key",
  "month",
  "note",
  "number",
  "organization",
  "pages",
  "publisher",
  "school",
  "series",
  "title",
  "volume",
  "year"
  ]


def bibtexparser_entry_to_papis(entry):
    """Convert keys of a bib entry in bibtexparser format to papis compatible
    format.

    :param entry: Dictionary with keys of bibtexparser format.
    :type  entry: dict
    :returns: Dictionary with keys of papis format.

    """
    result = dict()
    for key in entry.keys():
        if key == 'ID':
            result['ref'] = entry[key]
        elif key == 'ENTRYTYPE':
            result['type'] = entry[key]
        elif key == 'link':
            result['url'] = entry[key]
        else:
            result[key] = entry[key]
    return result


def bibtex_to_dict(bibtex):
    """
    Convert bibtex file to dict

    .. code:: python

        { type: "article ...", "ref": "example1960etAl", author:" ..."}

    :param bibtex: Bibtex file path or bibtex information in string format.
    :type  bibtex: str
    :returns: Dictionary with bibtex information with keys that bibtex
        formally recognizes.
    :rtype:  list
    """
    import bibtexparser
    # bibtexparser has too many debug messages to be useful
    logging.getLogger("bibtexparser.bparser").setLevel(logging.WARNING)
    global logger
    result = dict()
    if os.path.exists(bibtex):
        with open(bibtex) as fd:
            logger.debug("Reading in file %s" % bibtex)
            text = fd.read()
    else:
        text = bibtex
    logger.debug("Removing comments...")
    text = re.sub(r"%.*", "", text)
    logger.debug("Removing empty lines...")
    text = re.sub(r"^\s*$", "", text)
    entries = bibtexparser.loads(text).entries
    # Clean entries
    return [bibtexparser_entry_to_papis(entry) for entry in entries]

